@@ -115,9 +115,9 @@ void ArgumentSuggestion::addTo(multiset<ArgumentSuggestion> &suggestions, size_t
  * \class ArgumentReader
  * \brief The ArgumentReader class internally encapsulates the process of reading command line arguments.
  * \remarks
- * - For meaning of parameter see documentation of corresponding member variables.
+ * - For meaning of parameters see documentation of corresponding member variables.
  * - Results are stored in specified \a args and assigned sub arguments.
- * - This class is explicitely *not* part of the public API.
+ * - This class is explicitly *not* part of the public API.
  */
 
 /*!
@@ -185,7 +185,8 @@ bool ArgumentReader::read(ArgumentVector &args)
     // iterate through all argument denotations; loop might exit earlier when an denotation is unknown
     while (argv != end) {
         // check whether there are still values to read
-        if (values && lastArgInLevel->requiredValueCount() != Argument::varValueCount && values->size() < lastArgInLevel->requiredValueCount()) {
+        if (values && ((lastArgInLevel->requiredValueCount() != Argument::varValueCount) || (lastArgInLevel->flags() & Argument::Flags::Greedy))
+            && values->size() < lastArgInLevel->requiredValueCount()) {
             // read arg as value and continue with next arg
             values->emplace_back(argDenotation ? argDenotation : *argv);
             ++index;
@@ -397,7 +398,7 @@ bool ArgumentReader::read(ArgumentVector &args)
 /*!
  * \class Wrapper
  * \brief The Wrapper class is internally used print text which might needs to be wrapped preserving the indentation.
- * \remarks This class is explicitely *not* part of the public API.
+ * \remarks This class is explicitly *not* part of the public API.
  */
 
 ostream &operator<<(ostream &os, const Wrapper &wrapper)
@@ -444,10 +445,10 @@ inline bool notEmpty(const char *str)
  * \class Argument
  * \brief The Argument class is a wrapper for command line argument information.
  *
- * Instaces of the Argument class are used as definition when parsing command line
+ * Instances of the Argument class are used as definition when parsing command line
  * arguments. Arguments can be assigned to an ArgumentParser using
  * ArgumentParser::setMainArguments() and to another Argument instance using
- * Argument::setSecondaryArguments().
+ * Argument::setSubArguments().
  */
 
 /*!
@@ -499,6 +500,18 @@ const char *Argument::firstValue() const
     }
 }
 
+/*!
+ * \brief Returns the first value like Argument::firstValue() but returns \a fallback instead of nullptr if there's no value.
+ */
+const char *Argument::firstValueOr(const char *fallback) const
+{
+    if (const auto *const v = firstValue()) {
+        return v;
+    } else {
+        return fallback;
+    }
+}
+
 /*!
  * \brief Writes the name, the abbreviation and other information about the Argument to the give ostream.
  */
@@ -584,7 +597,7 @@ Argument *firstPresentUncombinableArg(const ArgumentVector &args, const Argument
 }
 
 /*!
- * \brief Sets the secondary arguments for this arguments.
+ * \brief Sets the secondary arguments for this argument.
  *
  * The given arguments will be considered as secondary arguments of this argument by the argument parser.
  * This means that the parser will complain if these arguments are given, but not this argument.
@@ -593,21 +606,47 @@ Argument *firstPresentUncombinableArg(const ArgumentVector &args, const Argument
  * The Argument does not take ownership. Do not destroy the given arguments as long as they are
  * used as secondary arguments.
  *
+ * \sa addSubArguments()
  * \sa secondaryArguments()
  * \sa addSecondaryArgument()
- * \sa hasSecondaryArguments()
+ * \sa hasSubArguments()
  */
-void Argument::setSubArguments(const ArgumentInitializerList &secondaryArguments)
+void Argument::setSubArguments(const ArgumentInitializerList &subArguments)
 {
     // remove this argument from the parents list of the previous secondary arguments
-    for (Argument *arg : m_subArgs) {
+    for (Argument *const arg : m_subArgs) {
         arg->m_parents.erase(remove(arg->m_parents.begin(), arg->m_parents.end(), this), arg->m_parents.end());
     }
-    // assign secondary arguments
-    m_subArgs.assign(secondaryArguments);
-    // add this argument to the parents list of the assigned secondary arguments
-    // and set the parser
-    for (Argument *arg : m_subArgs) {
+    // clear currently assigned args before adding new ones
+    m_subArgs.clear();
+    addSubArguments(subArguments);
+}
+
+/*!
+ * \brief Sets the secondary arguments for this argument.
+ *
+ * The given arguments will be considered as secondary arguments of this argument by the argument parser.
+ * This means that the parser will complain if these arguments are given, but not this argument.
+ * If secondary arguments are labeled as mandatory their parent is also mandatory.
+ *
+ * The Argument does not take ownership. Do not destroy the given arguments as long as they are
+ * used as secondary arguments.
+ *
+ * \sa setSubArguments()
+ * \sa secondaryArguments()
+ * \sa addSecondaryArgument()
+ * \sa hasSubArguments()
+ */
+void Argument::addSubArguments(const ArgumentInitializerList &subArguments)
+{
+    // append secondary arguments
+    const auto requiredCap = m_subArgs.size() + subArguments.size();
+    if (requiredCap < m_subArgs.capacity()) {
+        m_subArgs.reserve(requiredCap); // does insert this for us?
+    }
+    m_subArgs.insert(m_subArgs.end(), subArguments.begin(), subArguments.end());
+    // add this argument to the parents list of the assigned secondary arguments and set the parser
+    for (Argument *const arg : subArguments) {
         if (find(arg->m_parents.cbegin(), arg->m_parents.cend(), this) == arg->m_parents.cend()) {
             arg->m_parents.push_back(this);
         }
@@ -618,8 +657,8 @@ void Argument::setSubArguments(const ArgumentInitializerList &secondaryArguments
  * \brief Adds \a arg as a secondary argument for this argument.
  *
  * \sa secondaryArguments()
- * \sa setSecondaryArguments()
- * \sa hasSecondaryArguments()
+ * \sa setSubArguments()
+ * \sa hasSubArguments()
  */
 void Argument::addSubArgument(Argument *arg)
 {
@@ -908,7 +947,7 @@ void ArgumentParser::parseArgs(int argc, const char *const *argv, ParseArgumentB
         if (behavior & ParseArgumentBehavior::ExitOnFailure) {
             CMD_UTILS_START_CONSOLE;
             cerr << failure;
-            invokeExit(1);
+            invokeExit(EXIT_FAILURE);
         }
         throw;
     }
@@ -980,10 +1019,10 @@ void ArgumentParser::readArgs(int argc, const char *const *argv)
         throw ParseError(argsToString("The specified argument \"", *reader.argv, "\" is unknown.", suggestions));
     }
 
-    // print Bash completion and prevent the applicaton to continue with the regular execution
+    // print Bash completion and prevent the application to continue with the regular execution
     if (completionMode) {
         printBashCompletion(argc, argv, currentWordIndex, reader);
-        invokeExit(0);
+        invokeExit(EXIT_SUCCESS);
     }
 }
 
@@ -1059,7 +1098,7 @@ void ArgumentParser::verifyArgs(const ArgumentVector &args)
         assert(!arg->abbreviation() || find(abbreviations.cbegin(), abbreviations.cend(), arg->abbreviation()) == abbreviations.cend());
         abbreviations.push_back(arg->abbreviation());
         assert(!arg->name() || find_if(names.cbegin(), names.cend(), [arg](const char *name) { return !strcmp(arg->name(), name); }) == names.cend());
-        assert(arg->requiredValueCount() == 0 || arg->subArguments().size() == 0);
+        assert(arg->requiredValueCount() == 0 || arg->subArguments().size() == 0 || (arg->flags() & Argument::Flags::Greedy));
         names.emplace_back(arg->name());
     }
     for (const Argument *arg : args) {
@@ -1303,7 +1342,7 @@ void ArgumentParser::printBashCompletion(int argc, const char *const *argv, unsi
     if (argc && completionInfo.nextArgumentOrValue) {
         if (currentWordIndex < static_cast<unsigned int>(argc)) {
             opening = argv[currentWordIndex];
-            // For some reason completions for eg. "set --values disk=1 tag=a" are splitted so the
+            // For some reason completions for eg. "set --values disk=1 tag=a" are split so the
             // equation sign is an own argument ("set --values disk = 1 tag = a").
             // This is not how values are treated by the argument parser. Hence the opening
             // must be joined again. In this case only the part after the equation sign needs to be
@@ -1492,17 +1531,17 @@ void ArgumentParser::printBashCompletion(int argc, const char *const *argv, unsi
             if (dir.size() > 1 && (dir[dir.size() - 2] == '\"' || dir[dir.size() - 2] == '\'')) {
                 dir.erase(dir.size() - 2, 1);
             }
-            actualDir = move(dir);
+            actualDir = std::move(dir);
         }
         // determine the "file" part
         string file = fileName(unescapedOpening);
         if (file[0] == '\"' || file[0] == '\'') {
             file.erase(0, 1);
         }
-        if (file.size() > 1 && (file[dir.size() - 2] == '\"' || dir[file.size() - 2] == '\'')) {
+        if (file.size() > 1 && (file[file.size() - 2] == '\"' || file[file.size() - 2] == '\'')) {
             file.erase(file.size() - 2, 1);
         }
-        actualFile = move(file);
+        actualFile = std::move(file);
     }
 
     // -> completion for files and dirs
@@ -1559,7 +1598,7 @@ void ArgumentParser::printBashCompletion(int argc, const char *const *argv, unsi
         cout << "; compopt -o filenames";
     }
 
-    // ensure trailing whitespace is ommitted
+    // ensure trailing whitespace is omitted
     if (noWhitespace) {
         cout << "; compopt -o nospace";
     }
@@ -1569,7 +1608,7 @@ void ArgumentParser::printBashCompletion(int argc, const char *const *argv, unsi
 
 /*!
  * \brief Checks the constrains of the specified \a args.
- * \remarks Checks the contraints of sub arguments, too.
+ * \remarks Checks the constraints of sub arguments, too.
  */
 void ArgumentParser::checkConstraints(const ArgumentVector &args)
 {
@@ -1599,11 +1638,11 @@ void ArgumentParser::checkConstraints(const ArgumentVector &args)
                 continue;
             }
             stringstream ss(stringstream::in | stringstream::out);
-            ss << "Not all parameter for argument \"" << arg->name() << "\" ";
+            ss << "Not all parameters for argument \"" << arg->name() << "\" ";
             if (i) {
                 ss << " (" << (i + 1) << " occurrence) ";
             }
-            ss << "provided. You have to provide the following parameter:";
+            ss << "provided. You have to provide the following parameters:";
             size_t valueNamesPrint = 0;
             for (const auto &name : arg->m_valueNames) {
                 ss << ' ' << name;
@@ -1617,7 +1656,7 @@ void ArgumentParser::checkConstraints(const ArgumentVector &args)
             throw ParseError(ss.str());
         }
 
-        // check contraints of sub arguments recursively
+        // check constraints of sub arguments recursively
         checkConstraints(arg->m_subArgs);
     }
 }
@@ -1689,7 +1728,7 @@ HelpArgument::HelpArgument(ArgumentParser &parser)
  * \brief The NoColorArgument class allows to specify whether use of escape codes or similar technique to provide formatted output
  *        on the terminal should be enabled/disabled.
  *
- * This argument will either prevent or explicitely allow the use of escape codes or similar technique to provide formatted output
+ * This argument will either prevent or explicitly allow the use of escape codes or similar technique to provide formatted output
  * on the terminal. More explicitly, the argument will always allow to negate the default value of EscapeCodes::enabled which can be
  * configured at build time by setting the CMake variable ENABLE_ESCAPE_CODES_BY_DEFAULT.
  *
@@ -1716,31 +1755,18 @@ NoColorArgument::NoColorArgument()
 {
     setCombinable(true);
 
-    // set the environmentvariable: note that this is not directly used and just assigned for printing help
+    // set the environment variable (not directly used and just assigned for printing help)
     setEnvironmentVariable("ENABLE_ESCAPE_CODES");
 
-    // default-initialize EscapeCodes::enabled from environment variable
-    const char *envValue = getenv(environmentVariable());
-    if (!envValue) {
-        return;
+    // initialize EscapeCodes::enabled from environment variable
+    const auto escapeCodesEnabled = isEnvVariableSet(environmentVariable());
+    if (escapeCodesEnabled.has_value()) {
+        EscapeCodes::enabled = escapeCodesEnabled.value();
     }
-    for (; *envValue; ++envValue) {
-        switch (*envValue) {
-        case '0':
-        case ' ':
-            break;
-        default:
-            // enable escape codes if ENABLE_ESCAPE_CODES contains anything else than spaces or zeros
-            EscapeCodes::enabled = true;
-            return;
-        }
-    }
-    // disable escape codes if ENABLE_ESCAPE_CODES is empty or only contains spaces and zeros
-    EscapeCodes::enabled = false;
 }
 
 /*!
- * \brief Sets EscapeCodes::enabled according to the presense of the first instantiation of NoColorArgument.
+ * \brief Sets EscapeCodes::enabled according to the presence of the first instantiation of NoColorArgument.
  */
 void NoColorArgument::apply() const
 {
