@@ -9,6 +9,7 @@
 #include "../misc/parseerror.h"
 
 #include <cerrno>
+#include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
@@ -39,7 +40,7 @@ using namespace CppUtilities::EscapeCodes;
 namespace CppUtilities {
 
 /// \cond
-bool fileSystemItemExists(const string &path)
+static bool fileSystemItemExists(const string &path)
 {
 #ifdef PLATFORM_UNIX
     struct stat res;
@@ -54,7 +55,7 @@ bool fileSystemItemExists(const string &path)
 #endif
 }
 
-bool fileExists(const string &path)
+static bool fileExists(const string &path)
 {
 #ifdef PLATFORM_UNIX
     struct stat res;
@@ -69,7 +70,7 @@ bool fileExists(const string &path)
 #endif
 }
 
-bool dirExists(const string &path)
+static bool dirExists(const string &path)
 {
 #ifdef PLATFORM_UNIX
     struct stat res;
@@ -84,7 +85,7 @@ bool dirExists(const string &path)
 #endif
 }
 
-bool makeDir(const string &path)
+static bool makeDir(const string &path)
 {
 #ifdef PLATFORM_UNIX
     return mkdir(path.data(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) == 0;
@@ -173,11 +174,12 @@ TestApplication::TestApplication(int argc, const char *const *argv)
     // -> read TEST_FILE_PATH environment variable
     bool hasTestFilePathFromEnv;
     if (auto testFilePathFromEnv = readTestfilePathFromEnv(); (hasTestFilePathFromEnv = !testFilePathFromEnv.empty())) {
-        m_testFilesPaths.emplace_back(move(testFilePathFromEnv));
+        m_testFilesPaths.emplace_back(std::move(testFilePathFromEnv));
     }
     // -> find source directory
     if (auto testFilePathFromSrcDirRef = readTestfilePathFromSrcRef(); !testFilePathFromSrcDirRef.empty()) {
-        m_testFilesPaths.emplace_back(move(testFilePathFromSrcDirRef));
+        m_testFilesPaths.insert(m_testFilesPaths.end(), std::make_move_iterator(testFilePathFromSrcDirRef.begin()),
+            std::make_move_iterator(testFilePathFromSrcDirRef.end()));
     }
     // -> try testfiles directory in working directory
     m_testFilesPaths.emplace_back("./testfiles/");
@@ -233,16 +235,34 @@ TestApplication::~TestApplication()
  * 3. The subdirectory "testfiles" within the source directory, if it could be determined via "srcref"-file.
  * 4. The subdirectory "testfiles" within present working directory.
  */
-string TestApplication::testFilePath(const string &relativeTestFilePath) const
+std::string TestApplication::testFilePath(const std::string &relativeTestFilePath) const
 {
-    string path;
+    std::string path;
     for (const auto &testFilesPath : m_testFilesPaths) {
         if (fileExists(path = testFilesPath + relativeTestFilePath)) {
             return path;
         }
     }
-    throw runtime_error("The testfile \"" % relativeTestFilePath % "\" can not be located. Was looking under:"
-        + joinStrings(m_testFilesPaths, "\n", false, string(), relativeTestFilePath));
+    throw std::runtime_error("The test file \"" % relativeTestFilePath % "\" can not be located. Was looking under:\n"
+        + joinStrings(m_testFilesPaths, "\n", false, " - ", relativeTestFilePath));
+}
+
+/*!
+ * \brief Returns the full path of the test directory with the specified \a relativeTestDirPath.
+ *
+ * This is the same as TestApplication::testFilePath() but for directories. Checkout the documentation of
+ * TestApplication::testFilePath() for details about the lookup.
+ */
+std::string TestApplication::testDirPath(const std::string &relativeTestDirPath) const
+{
+    std::string path;
+    for (const auto &testFilesPath : m_testFilesPaths) {
+        if (dirExists(path = testFilesPath + relativeTestDirPath)) {
+            return path;
+        }
+    }
+    throw std::runtime_error("The test directory \"" % relativeTestDirPath % "\" can not be located. Was looking under:\n"
+        + joinStrings(m_testFilesPaths, "\n", false, " - ", relativeTestDirPath));
 }
 
 /*!
@@ -275,10 +295,11 @@ string TestApplication::workingCopyPathAs(
     const std::string &relativeTestFilePath, const std::string &relativeWorkingCopyPath, WorkingCopyMode mode) const
 {
     // ensure working directory is present
+    auto workingCopyPath = std::string();
     if (!dirExists(m_workingDir) && !makeDir(m_workingDir)) {
         cerr << Phrases::Error << "Unable to create working copy for \"" << relativeTestFilePath << "\": can't create working directory \""
              << m_workingDir << "\"." << Phrases::EndFlush;
-        return string();
+        return workingCopyPath;
     }
 
     // ensure subdirectory exists
@@ -301,26 +322,37 @@ string TestApplication::workingCopyPathAs(
             // fail otherwise
             cerr << Phrases::Error << "Unable to create working copy for \"" << relativeWorkingCopyPath << "\": can't create directory \""
                  << currentLevel << "\" (inside working directory)." << Phrases::EndFlush;
-            return string();
+            return workingCopyPath;
         }
     }
 
-    // just return the path if we don't want to actually create a copy
-    if (mode == WorkingCopyMode::NoCopy) {
-        return m_workingDir + relativeWorkingCopyPath;
+    workingCopyPath = m_workingDir + relativeWorkingCopyPath;
+    switch (mode) {
+    case WorkingCopyMode::NoCopy:
+        // just return the path if we don't want to actually create a copy
+        return workingCopyPath;
+    case WorkingCopyMode::Cleanup:
+        // ensure the file does not exist in cleanup mode
+        if (std::remove(workingCopyPath.data()) != 0 && errno != ENOENT) {
+            const auto error = std::strerror(errno);
+            cerr << Phrases::Error << "Unable to delete \"" << workingCopyPath << "\": " << error << Phrases::EndFlush;
+            workingCopyPath.clear();
+        }
+        return workingCopyPath;
+    default:;
     }
 
     // copy the file
-    const auto origFilePath(testFilePath(relativeTestFilePath));
-    auto workingCopyPath(m_workingDir + relativeWorkingCopyPath);
+    const auto origFilePath = testFilePath(relativeTestFilePath);
     size_t workingCopyPathAttempt = 0;
     NativeFileStream origFile, workingCopy;
     origFile.open(origFilePath, ios_base::in | ios_base::binary);
     if (origFile.fail()) {
         cerr << Phrases::Error << "Unable to create working copy for \"" << relativeTestFilePath
              << "\": an IO error occurred when opening original file \"" << origFilePath << "\"." << Phrases::EndFlush;
-        cerr << "error: " << strerror(errno) << endl;
-        return string();
+        cerr << "error: " << std::strerror(errno) << endl;
+        workingCopyPath.clear();
+        return workingCopyPath;
     }
     workingCopy.open(workingCopyPath, ios_base::out | ios_base::binary | ios_base::trunc);
     while (workingCopy.fail() && fileSystemItemExists(workingCopyPath)) {
@@ -333,7 +365,8 @@ string TestApplication::workingCopyPathAs(
         cerr << Phrases::Error << "Unable to create working copy for \"" << relativeTestFilePath
              << "\": an IO error occurred when opening target file \"" << workingCopyPath << "\"." << Phrases::EndFlush;
         cerr << "error: " << strerror(errno) << endl;
-        return string();
+        workingCopyPath.clear();
+        return workingCopyPath;
     }
     workingCopy << origFile.rdbuf();
     workingCopy.close();
@@ -344,7 +377,8 @@ string TestApplication::workingCopyPathAs(
     cerr << Phrases::Error << "Unable to create working copy for \"" << relativeTestFilePath << "\": ";
     if (origFile.fail()) {
         cerr << "an IO error occurred when reading original file \"" << origFilePath << "\"";
-        return string();
+        workingCopyPath.clear();
+        return workingCopyPath;
     }
     if (workingCopy.fail()) {
         if (origFile.fail()) {
@@ -353,7 +387,8 @@ string TestApplication::workingCopyPathAs(
         cerr << " an IO error occurred when writing to target file \"" << workingCopyPath << "\".";
     }
     cerr << "error: " << strerror(errno) << endl;
-    return string();
+    workingCopyPath.clear();
+    return workingCopyPath;
 }
 
 #ifdef PLATFORM_UNIX
@@ -361,8 +396,8 @@ string TestApplication::workingCopyPathAs(
  * \brief Executes an application with the specified \a args.
  * \remarks Provides internal implementation of execApp() and execHelperApp().
  */
-int execAppInternal(const char *appPath, const char *const *args, std::string &output, std::string &errors, bool suppressLogging, int timeout,
-    const std::string &newProfilingPath)
+static int execAppInternal(const char *appPath, const char *const *args, std::string &output, std::string &errors, bool suppressLogging, int timeout,
+    const std::string &newProfilingPath, bool enableSearchPath = false)
 {
     // print log message
     if (!suppressLogging) {
@@ -460,7 +495,12 @@ int execAppInternal(const char *appPath, const char *const *args, std::string &o
         }
 
         // -> execute application
-        execv(appPath, const_cast<char *const *>(args));
+        if (enableSearchPath) {
+            execvp(appPath, const_cast<char *const *>(args));
+
+        } else {
+            execv(appPath, const_cast<char *const *>(args));
+        }
         cerr << Phrases::Error << "Unable to execute \"" << appPath << "\": execv() failed" << Phrases::EndFlush;
         exit(-101);
     }
@@ -483,17 +523,17 @@ int TestApplication::execApp(const char *const *args, string &output, string &er
 
     // determine the path of the application to be tested
     const char *appPath = m_applicationPathArg.firstValue();
-    string fallbackAppPath;
+    auto fallbackAppPath = string();
     if (!appPath || !*appPath) {
         // try to find the path by removing "_tests"-suffix from own executable path
         // (the own executable path is the path of the test application and its name is usually the name of the application
         //  to be tested with "_tests"-suffix)
         const char *const testAppPath = m_parser.executable();
-        const size_t testAppPathLength = strlen(testAppPath);
+        const auto testAppPathLength = strlen(testAppPath);
         if (testAppPathLength > 6 && !strcmp(testAppPath + testAppPathLength - 6, "_tests")) {
             fallbackAppPath.assign(testAppPath, testAppPathLength - 6);
             appPath = fallbackAppPath.data();
-            // TODO: it would not hurt to verify whether "fallbackAppPath" actually exists and is executalbe
+            // TODO: it would not hurt to verify whether "fallbackAppPath" actually exists and is executable
         } else {
             throw runtime_error("Unable to execute application to be tested: no application path specified");
         }
@@ -501,27 +541,27 @@ int TestApplication::execApp(const char *const *args, string &output, string &er
 
     // determine new path for profiling output (to not override profiling output of parent and previous invocations)
     const auto newProfilingPath = [appPath] {
-        string newProfilingPath;
+        auto path = string();
         const char *const llvmProfileFile = getenv("LLVM_PROFILE_FILE");
         if (!llvmProfileFile) {
-            return newProfilingPath;
+            return path;
         }
         // replace eg. "/some/path/tageditor_tests.profraw" with "/some/path/tageditor0.profraw"
         const char *const llvmProfileFileEnd = strstr(llvmProfileFile, ".profraw");
         if (!llvmProfileFileEnd) {
-            return newProfilingPath;
+            return path;
         }
-        const string llvmProfileFileWithoutExtension(llvmProfileFile, llvmProfileFileEnd);
+        const auto llvmProfileFileWithoutExtension = string(llvmProfileFile, llvmProfileFileEnd);
         // extract application name from path
         const char *appName = strrchr(appPath, '/');
         appName = appName ? appName + 1 : appPath;
         // concat new path
-        newProfilingPath = argsToString(llvmProfileFileWithoutExtension, '_', appName, invocationCount, ".profraw");
+        path = argsToString(llvmProfileFileWithoutExtension, '_', appName, invocationCount, ".profraw");
         // append path to profiling list file
         if (const char *const profrawListFile = getenv("LLVM_PROFILE_LIST_FILE")) {
-            ofstream(profrawListFile, ios_base::app) << newProfilingPath << endl;
+            ofstream(profrawListFile, ios_base::app) << path << endl;
         }
-        return newProfilingPath;
+        return path;
     }();
 
     return execAppInternal(appPath, args, output, errors, suppressLogging, timeout, newProfilingPath);
@@ -538,6 +578,22 @@ int execHelperApp(const char *appPath, const char *const *args, std::string &out
 {
     return execAppInternal(appPath, args, output, errors, suppressLogging, timeout, string());
 }
+
+/*!
+ * \brief Executes an application with the specified \a args.
+ *
+ * Searches for the location of \a appName among the directories specified by the PATH environment variable.
+ *
+ * \remarks
+ * - Intended to invoke helper applications (eg. to setup test files). Use execApp() and TestApplication::execApp() to
+ *   invoke the application to be tested itself.
+ * - Currently only supported under UNIX.
+ */
+int execHelperAppInSearchPath(
+    const char *appName, const char *const *args, std::string &output, std::string &errors, bool suppressLogging, int timeout)
+{
+    return execAppInternal(appName, args, output, errors, suppressLogging, timeout, string(), true);
+}
 #endif // PLATFORM_UNIX
 
 /*!
@@ -557,11 +613,12 @@ string TestApplication::readTestfilePathFromEnv()
  * \remarks That file is supposed to contain the path the the source directory. It is supposed to be stored by the build system in the
  *          same directory as the test executable. The CMake modules contained of these utilities ensure that's the case.
  */
-string TestApplication::readTestfilePathFromSrcRef()
+std::vector<std::string> TestApplication::readTestfilePathFromSrcRef()
 {
     // find the path of the current executable on platforms supporting "/proc/self/exe"; otherwise assume the current working directory
     // is the executable path
-    std::string binaryPath;
+    auto res = std::vector<std::string>();
+    auto binaryPath = std::string();
 #if defined(CPP_UTILITIES_USE_STANDARD_FILESYSTEM) && defined(PLATFORM_UNIX)
     try {
         binaryPath = std::filesystem::read_symlink("/proc/self/exe").parent_path();
@@ -570,28 +627,32 @@ string TestApplication::readTestfilePathFromSrcRef()
         cerr << Phrases::Warning << "Unable to detect binary path for finding \"srcdirref\": " << e.what() << Phrases::EndFlush;
     }
 #endif
+    const auto srcdirrefPath = binaryPath + "srcdirref";
     try {
         // read "srcdirref" file which should contain the path of the source directory
-        auto srcDirContent(readFile(binaryPath + "srcdirref", 2 * 1024));
+        const auto srcDirContent = readFile(srcdirrefPath, 2 * 1024);
         if (srcDirContent.empty()) {
             cerr << Phrases::Warning << "The file \"srcdirref\" is empty." << Phrases::EndFlush;
-            return string();
+            return res;
         }
-        srcDirContent += "/testfiles/";
 
-        // check whether the referenced source directory contains a "testfiles" directory
-        if (!dirExists(srcDirContent)) {
-            cerr << Phrases::Warning
-                 << "The source directory referenced by the file \"srcdirref\" does not contain a \"testfiles\" directory or does not exist."
-                 << Phrases::End << "Referenced source directory: " << srcDirContent << endl;
-            return string();
+        // check whether the referenced source directories contain a "testfiles" directory
+        const auto srcPaths = splitStringSimple<std::vector<std::string_view>>(srcDirContent, "\n");
+        for (const auto &srcPath : srcPaths) {
+            auto testfilesPath = argsToString(srcPath, "/testfiles/");
+            if (dirExists(testfilesPath)) {
+                res.emplace_back(std::move(testfilesPath));
+            } else {
+                cerr << Phrases::Warning
+                     << "The source directory referenced by the file \"srcdirref\" does not contain a \"testfiles\" directory or does not exist."
+                     << Phrases::End << "Referenced source directory: " << testfilesPath << endl;
+            }
         }
-        return srcDirContent;
+        return res;
 
-    } catch (const std::ios_base::failure &) {
-        cerr << Phrases::Warning << "The file \"srcdirref\" can not be opened. It likely just doesn't exist in the working directory."
-             << Phrases::EndFlush;
+    } catch (const std::ios_base::failure &e) {
+        cerr << Phrases::Warning << "The file \"" << srcdirrefPath << "\" can not be opened: " << e.what() << Phrases::EndFlush;
     }
-    return string();
+    return res;
 }
 } // namespace CppUtilities
